use nom::error::VerboseError;
use nom::IResult;

use super::list::List;

type Matcher = dyn for<'s> Fn(&'s str) -> IResult<&'s str, &'s str, VerboseError<&'s str>>;

pub struct ContextTree<'r> {
    tree: List<ContextElement<'r>>,
}

impl<'r> ContextTree<'r> {
    pub fn new() -> Self {
        ContextTree { tree: List::new() }
    }
}

pub enum ContextElement<'r> {
    FailMatcherNode(FailMatcherNode<'r>),
    PreviousElementNode(PreviousElementNode<'r>),
}

pub struct FailMatcherNode<'r> {
    pub fail_matcher: ChainBehavior<'r>,
}

pub struct PreviousElementNode<'r> {
    pub dummy: &'r str,
}

#[derive(Clone)]
pub enum ChainBehavior<'r> {
    AndParent(Option<&'r Matcher>),
    IgnoreParent(Option<&'r Matcher>),
}
